// efeito das particulas
float4x4 World : WORLD;
float4x4 View;
float4x4 Projection;

texture ColorMap;

sampler ColorMapSampler = sampler_state
{
	Texture = <ColorMap>;
	magfilter = LINEAR;
	minfilter = LINEAR;
	mipfilter = LINEAR;
	//addressU = mirror;
	//addressV = mirror;
};

struct VertexInput
{
	float4 Position : POSITION0;
	float4 Color : COLOR0;
	float Size : PSIZE0;
};

struct VertexOutput
{
	float4 Position : POSITION0;
	float Size : PSIZE0;
	float4 Color : COLOR0;
	float4 TexCoords : TEXCOORD0;
};

VertexOutput vertexShader (VertexInput input)
{
	VertexOutput output;
	float4x4 worldViewProjection = mul(mul(World, View), Projection);
	output.Position = mul(input.Position, worldViewProjection);
	output.Color = input.Color;
	output.Size = input.Size;
	output.TexCoords = (1.0f, 1.0f);
	return output;
}

struct PixelInput
{
	//float3 Position : POSITION0;
#ifdef XBOX360
	float4 TexCoords : SPRITETEXCOORD0;
#else
	float2 TexCoords : TEXCOORD0;
#endif
	float4 Color : COLOR0;
};

float4 pixelShader(PixelInput input) : COLOR0
{
	float2 texCoords;
#ifdef XBOX360
	texCoords = abs(input.TexCoords.zw);
#else
	texCoords = input.TexCoords.xy;
#endif
	return (tex2D(ColorMapSampler, texCoords) * input.Color);
	//return (1, 0, 0, input.Color.a);
}

technique ParticleEffect
{
	pass P0
	{
		sampler[0] = (ColorMapSampler);
		PointSpriteEnable = true; // needed for point sprite
		PointSize_Min = 0; // enable scaling to 0 on xbox
		AlphaBlendEnable = true; // enable transparency
		SrcBlend = SrcAlpha; // turn off transparent pixels
		DestBlend = One; // shiny blending
		ZWriteEnable = true; // disable 3D rendering

		VertexShader = compile vs_1_1 vertexShader();
		PixelShader = compile ps_1_1 pixelShader();
	}
}

struct VertexShaderInput
{
	float4 Position : POSITION0;
};

struct VertexShaderOutput
{
	float4 Position : POSITION0;
};

VertexShaderOutput VertexShaderFunction(VertexShaderInput input)
{
	VertexShaderOutput output;
	float4x4 worldViewProjection = mul(mul(World, View), Projection);
	output.Position = mul(input.Position, worldViewProjection);
	return output;
}

float4 PixelShaderFunction(PixelInput input) : COLOR0
{
	return input.Color;
}

technique Default
{
	pass Pass1
	{
		VertexShader = compile vs_1_1 VertexShaderFunction( );
		PixelShader = compile ps_1_1 PixelShaderFunction( );
	}
}
