﻿#region Using statementes
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using RhythmComet.Physics;
using FMOD;
using RhythmComet.Services;
using RhythmComet.Particles;
using System.Collections.Generic;

#if !XBOX
using System;
using System.Threading;
#endif

#endregion

namespace RhythmComet
{
    public interface IStartMenuState : IGameState { }

    public sealed class StartMenuState : GameState, IStartMenuState
    {
        #region Fields and properties

        private Texture2D background;
        private Vector2 backgroundPosition;
#if !XBOX
        private string[] menuOptions = new string[5] { "Play Game", "Choose song", "Instructions", "Credits", "Exit" };
        private bool songReady = false;
        private bool isXbox = false;
#else
        private string[] menuOptions = new string[4] { "Play Game", "Instructions", "Credits", "Exit" };
        private bool songReady = true;
        private bool isXbox = true;
#endif
        private SpriteFont font;
        private SpriteFont font2;
        private int optionSelected;
        
        #endregion

        #region Initializing
        public StartMenuState(Game game)
            : base(game)
        {
            game.Services.AddService(typeof(IStartMenuState), this);
            optionSelected = 1;
        }

        protected override void LoadContent()
        {
            background = game.Content.Load<Texture2D>("sprites/background");
            font = game.Content.Load<SpriteFont>(@"fonts\GridExercise");
            backgroundPosition = new Vector2((game.GraphicsDevice.Viewport.Width / 2 - background.Width / 2),
                (game.GraphicsDevice.Viewport.Height / 2 - background.Height / 2));
            font2 = game.Content.Load<SpriteFont>(@"fonts\BerlinSmallCaps");
        }

        #endregion

        #region Update and Draw
        public override void Update(GameTime gameTime)
        {
            if (Input.GamePadState.Buttons.Back == ButtonState.Pressed)
                game.Exit();
#if !XBOX
            if (Input.WasPressed(Buttons.Start) || Input.WasPressed(Buttons.A) || Input.WasPressed(Keys.Enter))
#else
            if (Input.WasPressed(Buttons.Start) || Input.WasPressed(Buttons.A))
#endif
            {
                if (optionSelected == 0 && songReady)
                {
                    songReady = false;
                    game.ObstacleManager.Reset();
#if !XBOX
                    game.Player.Initialize();
#endif
                    GameManager.PushState(game.PlayingState.Value);
                }
                else if (optionSelected == 1 && !isXbox)
                {
#if !XBOX
                    Thread box = new Thread(new ThreadStart(OpenSong));
                    box.SetApartmentState(ApartmentState.STA);
                    box.Start();
#endif
                }
                else if ((optionSelected == 2 && !isXbox) || (optionSelected == 1 && isXbox))
                {
                    GameManager.PushState(game.InstructionsState.Value);
                }
                else if ((optionSelected == 4 && !isXbox) || (optionSelected == 3 && isXbox))
                {
                    Game.Exit();
                }
            }
#if !XBOX
            if (Input.WasPressed(Keys.Down) || Input.WasPressed(Buttons.DPadDown) || Input.WasPressed(Buttons.LeftThumbstickDown))
#else
            if (Input.WasPressed(Buttons.DPadDown) || Input.WasPressed(Buttons.LeftThumbstickDown))
#endif
            {
                optionSelected = (optionSelected + 1) % menuOptions.Length;
            }

#if !XBOX
            if (Input.WasPressed(Keys.Up) || Input.WasPressed(Buttons.DPadUp) || Input.WasPressed(Buttons.LeftThumbstickUp))
#else
            if (Input.WasPressed(Buttons.DPadUp) || Input.WasPressed(Buttons.LeftThumbstickUp))
#endif
            {
                optionSelected = optionSelected - 1;
            }

            if ((optionSelected < 1 && !songReady) || (optionSelected < 0))
                optionSelected = menuOptions.Length - 1;

            base.Update(gameTime);
        }

        public override void Draw(GameTime gameTime)
        {
            Color color;
            Vector2 position = new Vector2(100, 80);
            
            game.GraphicsDevice.Clear(Color.Black);

            game.SpriteBatch.Begin();

            game.SpriteBatch.Draw(background, backgroundPosition, Color.White);

            if (songReady)
            {
#if !XBOX
                DrawID3info();
#endif
                if (optionSelected == 0)
                    color = Color.Red;
                else
                    color = Color.White;

                game.SpriteBatch.DrawString(font, menuOptions[0], position, color);
                position.Y += 20;
            }

            for (int i = 1; i < menuOptions.Length; i++)
            {
                if (optionSelected == i)
                    color = Color.Red;
                else
                    color = Color.White;

                game.SpriteBatch.DrawString(font, menuOptions[i], position, color);
                 position.Y += 20;
            }
            game.SpriteBatch.End();

            base.Draw(gameTime);
        }

#if !XBOX
        public void DrawID3info()
        {
            Vector2 infoPosition = new Vector2(game.SpriteBatch.GraphicsDevice.Viewport.Width / 2 - 100, game.SpriteBatch.GraphicsDevice.Viewport.Height / 2 + 100);
            //Vector2 infoPosition = new Vector2(100, 300);
            game.SpriteBatch.DrawString(font2, "Artist:", infoPosition, Color.White);
            game.SpriteBatch.DrawString(font2, game.Player.Artist, new Vector2(infoPosition.X + 100, infoPosition.Y), Color.White);
            infoPosition.Y += 20;
            game.SpriteBatch.DrawString(font2, "Title:", infoPosition, Color.White);
            game.SpriteBatch.DrawString(font2, game.Player.Title, new Vector2(infoPosition.X + 100, infoPosition.Y), Color.White);
            infoPosition.Y += 20;
            game.SpriteBatch.DrawString(font2, "Album:", infoPosition, Color.White);
            game.SpriteBatch.DrawString(font2, game.Player.Album, new Vector2(infoPosition.X + 100, infoPosition.Y), Color.White);
            infoPosition.Y += 20;
        }
#endif

        #endregion

        #region Helper methods
#if !XBOX
        private void OpenSong()
        {
            System.Windows.Forms.OpenFileDialog fileDialog = new System.Windows.Forms.OpenFileDialog();

            fileDialog.Title = "Choose your song";

            fileDialog.Filter = "Music Files (*.mp3)|*.mp3";

            if (fileDialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                game.Player.SongFilename = fileDialog.FileName;
                game.Player.Initialize();
                game.Player.ReadTags();
                songReady = true;
                optionSelected = 0;
            }
        }
#endif
        #endregion
    }
}
