/* ****************************************************************************
* Copyright (c) TCC - 2009 IME - USP. 
*
* This source code is subject to terms and conditions of the Microsoft Public License. 
* A copy of the license can be found in http://msdn.microsoft.com/en-us/library/dd228915.aspx. 
* By using this source code in any fashion, you are agreeing to be bound by the terms of the 
* Microsoft Public License.
*
* You must not remove this notice, or any other, from this software.
* ***************************************************************************/

namespace SoundLibrary
{
    internal class Header
    {
        #region Private methods
        private void InitBlock()
        {
            syncmode = Bitstream.INITIAL_SYNC;
        }
        #endregion

        #region Properties
        public virtual int SyncHeader
        {
            get
            {
                return _headerstring;
            }
        }
        #endregion

        public static readonly int[][] frequencies = { new int[] { 22050, 24000, 16000, 1 }, new int[] { 44100, 48000, 32000, 1 }, new int[] { 11025, 12000, 8000, 1 } }; // SZD: MPEG25

        public const int MPEG2_LSF = 0;
        public const int MPEG25_LSF = 2;

        public const int MPEG1 = 1;

        public const int STEREO = 0;
        public const int JOINT_STEREO = 1;
        public const int DUAL_CHANNEL = 2;
        public const int SINGLE_CHANNEL = 3;
        public const int FOURTYFOUR_POINT_ONE = 0;
        public const int FOURTYEIGHT = 1;
        public const int THIRTYTWO = 2;

        private int h_layer, h_protection_bit, h_bitrate_index, h_padding_bit, h_mode_extension;
        private int h_version;
        private int h_mode;
        private int h_sample_frequency;
        private int h_number_of_subbands, h_intensity_stereo_bound;
        private bool h_copyright, h_original;
        private sbyte syncmode;
        private Crc16 crc;

        public short checksum;
        public int framesize;
        public int nSlots;

        private int _headerstring = -1;

        #region Contructors
        internal Header()
        {
            InitBlock();
        }
        #endregion

        #region Public Methods
        public override System.String ToString()
        {
            System.Text.StringBuilder buffer = new System.Text.StringBuilder(200);
            buffer.Append("Layer ");
            buffer.Append(Layer_string());
            buffer.Append(" frame ");
            buffer.Append(Mode_string());
            buffer.Append(' ');
            buffer.Append(Version_string());
            if (!Checksums())
            {
                buffer.Append(" no");
            }
            buffer.Append(" checksums");
            buffer.Append(' ');
            buffer.Append(Sample_frequency_string());
            buffer.Append(',');
            buffer.Append(' ');
            buffer.Append(Bitrate_string());

            System.String s = buffer.ToString();
            return s;
        }

        public int Version()
        {
            return h_version;
        }

        public int Layer()
        {
            return h_layer;
        }

        public int Bitrate_index()
        {
            return h_bitrate_index;
        }

        public int Sample_frequency()
        {
            return h_sample_frequency;
        }

        public int Frequency()
        {
            return frequencies[h_version][h_sample_frequency];
        }

        public int Mode()
        {
            return h_mode;
        }

        public bool Checksums()
        {
            if (h_protection_bit == 0)
                return true;
            else
                return false;
        }

        public bool Copyright()
        {
            return h_copyright;
        }

        public bool Original()
        {
            return h_original;
        }

        public bool Checksum_ok()
        {
            return (checksum == crc.Checksum());
        }

        public bool Padding()
        {
            if (h_padding_bit == 0)
                return false;
            else
                return true;
        }

        public int Slots()
        {
            return nSlots;
        }

        public int Mode_extension()
        {
            return h_mode_extension;
        }

        public int Calculate_framesize()
        {

            if (h_layer == 1)
            {
                framesize = (12 * bitrates[h_version][0][h_bitrate_index]) / frequencies[h_version][h_sample_frequency];
                if (h_padding_bit != 0)
                    framesize++;
                framesize <<= 2;
                nSlots = 0;
            }
            else
            {
                framesize = (144 * bitrates[h_version][h_layer - 1][h_bitrate_index]) / frequencies[h_version][h_sample_frequency];
                if (h_version == MPEG2_LSF || h_version == MPEG25_LSF)
                    framesize >>= 1;
                if (h_padding_bit != 0)
                    framesize++;
                if (h_layer == 3)
                {
                    if (h_version == MPEG1)
                    {
                        nSlots = framesize - ((h_mode == SINGLE_CHANNEL) ? 17 : 32) - ((h_protection_bit != 0) ? 0 : 2) - 4; // header size
                    }
                    else
                    {
                        nSlots = framesize - ((h_mode == SINGLE_CHANNEL) ? 9 : 17) - ((h_protection_bit != 0) ? 0 : 2) - 4; // header size
                    }
                }
                else
                {
                    nSlots = 0;
                }
            }
            framesize -= 4;
            return framesize;
        }

        public int Max_number_of_frames(int streamsize)
        {
            if ((framesize + 4 - h_padding_bit) == 0)
                return 0;
            else
                return (streamsize / (framesize + 4 - h_padding_bit));
        }

        public int Min_number_of_frames(int streamsize)
        {
            if ((framesize + 5 - h_padding_bit) == 0)
                return 0;
            else
                return (streamsize / (framesize + 5 - h_padding_bit));
        }

        public float Ms_per_frame()
        {
            float[][] ms_per_frame_array = { new float[] { 8.707483f, 8.0f, 12.0f }, new float[] { 26.12245f, 24.0f, 36.0f }, new float[] { 26.12245f, 24.0f, 36.0f } };
            return (ms_per_frame_array[h_layer - 1][h_sample_frequency]);
        }

        public float Total_ms(int streamsize)
        {
            return (Max_number_of_frames(streamsize) * Ms_per_frame());
        }


        public string Layer_string()
        {
            switch (h_layer)
            {

                case 1:
                    return "I";

                case 2:
                    return "II";

                case 3:
                    return "III";
            }
            return null;
        }

        public string Bitrate_string()
        {
            return bitrate_str[h_version][h_layer - 1][h_bitrate_index];
        }

        public string Sample_frequency_string()
        {
            switch (h_sample_frequency)
            {

                case THIRTYTWO:
                    if (h_version == MPEG1)
                    {
                        return "32 kHz";
                    }
                    else if (h_version == MPEG2_LSF)
                    {
                        return "16 kHz";
                    }
                    else
                    {
                        return "8 kHz";
                    }

                case FOURTYFOUR_POINT_ONE:
                    if (h_version == MPEG1)
                    {
                        return "44.1 kHz";
                    }
                    else if (h_version == MPEG2_LSF)
                    {
                        return "22.05 kHz";
                    }
                    else
                    {
                        return "11.025 kHz";
                    }

                case FOURTYEIGHT:
                    if (h_version == MPEG1)
                    {
                        return "48 kHz";
                    }
                    else if (h_version == MPEG2_LSF)
                    {
                        return "24 kHz";
                    }
                    else
                    {
                        return "12 kHz";
                    }
            }
            return (null);
        }

        public string Mode_string()
        {
            switch (h_mode)
            {

                case STEREO:
                    return "Stereo";

                case JOINT_STEREO:
                    return "Joint stereo";

                case DUAL_CHANNEL:
                    return "Dual channel";

                case SINGLE_CHANNEL:
                    return "Single channel";
            }
            return null;
        }

        public string Version_string()
        {
            switch (h_version)
            {

                case MPEG1:
                    return "MPEG-1";

                case MPEG2_LSF:
                    return "MPEG-2 LSF";

                case MPEG25_LSF:
                    return "MPEG-2.5 LSF";
            }
            return (null);
        }

        public int Number_of_subbands()
        {
            return h_number_of_subbands;
        }

        public int Intensity_stereo_bound()
        {
            return h_intensity_stereo_bound;
        }

        #endregion

        #region Internal methods
        internal void read_header(Bitstream stream, Crc16[] crcp)
        {
            int headerstring;
            int channel_bitrate;

            bool sync = false;

            do
            {

                headerstring = stream.SyncHeader(syncmode);
                _headerstring = headerstring;

                if (syncmode == Bitstream.INITIAL_SYNC)
                {
                    h_version = ((SupportClass.URShift(headerstring, 19)) & 1);
                    if (((SupportClass.URShift(headerstring, 20)) & 1) == 0)
                        if (h_version == MPEG2_LSF)
                            h_version = MPEG25_LSF;
                        else
                        {
                            ////throw stream.newBitstreamException(javazoom.jl.decoder.BitstreamErrors_Fields.UNKNOWN_ERROR);
                        }


                    if ((h_sample_frequency = ((SupportClass.URShift(headerstring, 10)) & 3)) == 3)
                    {
                        ////throw stream.newBitstreamException(javazoom.jl.decoder.BitstreamErrors_Fields.UNKNOWN_ERROR);
                    }
                }

                h_layer = 4 - (SupportClass.URShift(headerstring, 17)) & 3;
                h_protection_bit = (SupportClass.URShift(headerstring, 16)) & 1;
                h_bitrate_index = (SupportClass.URShift(headerstring, 12)) & 0xF;
                h_padding_bit = (SupportClass.URShift(headerstring, 9)) & 1;
                h_mode = ((SupportClass.URShift(headerstring, 6)) & 3);
                h_mode_extension = (SupportClass.URShift(headerstring, 4)) & 3;
                if (h_mode == JOINT_STEREO)
                {
                    h_intensity_stereo_bound = (h_mode_extension << 2) + 4;
                }
                else
                {
                    h_intensity_stereo_bound = 0;
                }

                if (((SupportClass.URShift(headerstring, 3)) & 1) == 1)
                {
                    h_copyright = true;
                }

                if (((SupportClass.URShift(headerstring, 2)) & 1) == 1)
                {
                    h_original = true;
                }

                if (h_layer == 1)
                {
                    h_number_of_subbands = 32;
                }
                else
                {
                    channel_bitrate = h_bitrate_index;
                    if (h_mode != SINGLE_CHANNEL)
                        if (channel_bitrate == 4)
                            channel_bitrate = 1;
                        else
                            channel_bitrate -= 4;

                    if ((channel_bitrate == 1) || (channel_bitrate == 2))
                        if (h_sample_frequency == THIRTYTWO)
                            h_number_of_subbands = 12;
                        else
                            h_number_of_subbands = 8;
                    else if ((h_sample_frequency == FOURTYEIGHT) || ((channel_bitrate >= 3) && (channel_bitrate <= 5)))
                        h_number_of_subbands = 27;
                    else
                        h_number_of_subbands = 30;
                }
                if (h_intensity_stereo_bound > h_number_of_subbands)
                    h_intensity_stereo_bound = h_number_of_subbands;
                Calculate_framesize();

                stream.Read_frame_data(framesize);

                if (stream.IsSyncCurrentPosition(syncmode))
                {
                    if (syncmode == Bitstream.INITIAL_SYNC)
                    {
                        syncmode = Bitstream.STRICT_SYNC;
                        stream.Set_syncword(headerstring & unchecked((int)0xFFF80CC0));
                    }
                    sync = true;
                }
                else
                {
                    stream.UnreadFrame();
                }
            }
            while (!sync);

            stream.Parse_frame();

            if (h_protection_bit == 0)
            {
                checksum = (short)stream.Get_bits(16);
                if (crc == null)
                    crc = new Crc16();
                crc.Add_bits(headerstring, 16);
                crcp[0] = crc;
            }
            else
                crcp[0] = null;
            if (h_sample_frequency == FOURTYFOUR_POINT_ONE)
            {
                ////if (offset == null)
                ////{
                ////int max = max_number_of_frames(stream);
                ////offset = new int[max];
                ////for(int i=0; i<max; i++) offset[i] = 0;
                ////}
                ////// Bizarre, y avait ici une acollade ouvrante
                ////int cf = stream.current_frame();
                ////int lf = stream.last_frame();
                ////if ((cf > 0) && (cf == lf))
                ////{
                ////offset[cf] = offset[cf-1] + h_padding_bit;
                ////}
                ////else
                ////{
                ////offset[0] = h_padding_bit;
                ////}
            }
        }
        #endregion

        public static readonly int[][][] bitrates = { new int[][] { new int[] { 0, 32000, 48000, 56000, 64000, 80000, 96000, 112000, 128000, 144000, 160000, 176000, 192000, 224000, 256000, 0 }, new int[] { 0, 8000, 16000, 24000, 32000, 40000, 48000, 56000, 64000, 80000, 96000, 112000, 128000, 144000, 160000, 0 }, new int[] { 0, 8000, 16000, 24000, 32000, 40000, 48000, 56000, 64000, 80000, 96000, 112000, 128000, 144000, 160000, 0 } }, new int[][] { new int[] { 0, 32000, 64000, 96000, 128000, 160000, 192000, 224000, 256000, 288000, 320000, 352000, 384000, 416000, 448000, 0 }, new int[] { 0, 32000, 48000, 56000, 64000, 80000, 96000, 112000, 128000, 160000, 192000, 224000, 256000, 320000, 384000, 0 }, new int[] { 0, 32000, 40000, 48000, 56000, 64000, 80000, 96000, 112000, 128000, 160000, 192000, 224000, 256000, 320000, 0 } }, new int[][] { new int[] { 0, 32000, 48000, 56000, 64000, 80000, 96000, 112000, 128000, 144000, 160000, 176000, 192000, 224000, 256000, 0 }, new int[] { 0, 8000, 16000, 24000, 32000, 40000, 48000, 56000, 64000, 80000, 96000, 112000, 128000, 144000, 160000, 0 }, new int[] { 0, 8000, 16000, 24000, 32000, 40000, 48000, 56000, 64000, 80000, 96000, 112000, 128000, 144000, 160000, 0 } } };



        public static readonly System.String[][][] bitrate_str = {new System.String[][]{new System.String[]{"free format", "32 kbit/s", "48 kbit/s", "56 kbit/s", "64 kbit/s", "80 kbit/s", "96 kbit/s", "112 kbit/s", "128 kbit/s", "144 kbit/s", "160 kbit/s", "176 kbit/s", "192 kbit/s", "224 kbit/s", "256 kbit/s", "forbidden"}, new System.String[]{"free format", "8 kbit/s", "16 kbit/s", "24 kbit/s", "32 kbit/s", "40 kbit/s", "48 kbit/s", "56 kbit/s", "64 kbit/s", "80 kbit/s", "96 kbit/s", "112 kbit/s", "128 kbit/s", "144 kbit/s", "160 kbit/s", "forbidden"}, new System.String[]{"free format", "8 kbit/s", "16 kbit/s", "24 kbit/s", "32 kbit/s", "40 kbit/s", "48 kbit/s", "56 kbit/s", "64 kbit/s", "80 kbit/s", "96 kbit/s", "112 kbit/s", "128 kbit/s", "144 kbit/s", "160 kbit/s", "forbidden"}}, new System.String[][]{new System.String[]{"free format", "32 kbit/s", "64 kbit/s", "96 kbit/s", "128 kbit/s", "160 kbit/s", "192 kbit/s", "224 kbit/s", "256 kbit/s", "288 kbit/s", "320 kbit/s", "352 kbit/s", "384 kbit/s", "416 kbit/s", "448 kbit/s", "forbidden"}, new System.String[]{"free format", "32 kbit/s", "48 kbit/s", "56 kbit/s", "64 kbit/s", "80 kbit/s", "96 kbit/s", "112 kbit/s", "128 kbit/s", "160 kbit/s", "192 kbit/s", "224 kbit/s", "256 kbit/s", "320 kbit/s", "384 kbit/s", "forbidden"}, new System.String[]{"free format", "32 kbit/s", "40 kbit/s", "48 kbit/s", "56 kbit/s", "64 kbit/s", "80 kbit/s", "96 kbit/s", "112 kbit/s", "128 kbit/s", "160 kbit/s", "192 kbit/s", "224 kbit/s", "256 kbit/s", "320 kbit/s", "forbidden"}}, new System.String[][]{new System.String[]{"free format", "32 kbit/s", "48 kbit/s", "56 kbit/s", "64 kbit/s", "80 kbit/s", "96 kbit/s", "112 kbit/s", "128 kbit/s", "144 kbit/s", "160 kbit/s", "176 kbit/s", "192 kbit/s", "224 kbit/s", "256 kbit/s", "forbidden"}, new System.String[]{"free format", "8 kbit/s", "16 kbit/s", "24 kbit/s", "32 kbit/s", "40 kbit/s", "48 kbit/s", "56 kbit/s", "64 kbit/s", "80 kbit/s", "96 kbit/s", "112 kbit/s", "128 kbit/s", "144 kbit/s", "160 kbit/s", "forbidden"}, new System.
String[]{"free format", "8 kbit/s", "16 kbit/s", "24 kbit/s", "32 kbit/s", "40 kbit/s", "48 kbit/s", "56 kbit/s", "64 kbit/s", "80 kbit/s", "96 kbit/s", "112 kbit/s", "128 kbit/s", "144 kbit/s", "160 kbit/s", "forbidden"}}};


    }
}